#!/usr/bin/python2.4
# -*- coding: utf-8 -*-

import os
import string
from machine_config import *
from machine_global import *
from vector import *
from base_import import BaseImport
from dxf import DXF
from cf2 import CF2
from plt import PLT
from iso import ISO
from generator import Gen

PREVIEW_FRAME = 10

class Converter(BaseImport, DXF, CF2, PLT, ISO, Gen):
    def __init__(self, root, machine):
        self.root_window = root
        self.machine = machine
        BaseImport.__init__(self)
        DXF.__init__(self)
        CF2.__init__(self)
        PLT.__init__(self)
        ISO.__init__(self)
        Gen.__init__(self)

        self.open_dir = ""
        self.source = []
        self.import_layers = []
        self.raw_layer = None
        self.layers = []
        self.gcode = []
        self.coords = []
        self.file_name = ""
        self.file_format = InputFormat.NONE
        self.file_loaded = False
        self.canvas_items = []
        self.canvas_cross = []

    def generate(self):
        self.gcode = []
        self.coords = []
        if self.file_format in [InputFormat.DXF, InputFormat.CF2, InputFormat.PLT]:
            self.generate_plasma()
#            self.generate_gcode()
        elif self.file_format in [InputFormat.ISO]:
            self.generate_plasma()
#            self.generate_gcode()

    def open_file(self):
        if os.system("python ../../unikat/axis/open.py %s 1" % self.open_dir) != 0:
            return False
        f = open("/tmp/name.axis")
        self.file_name = f.readline()
        f.close()
        self.file_format = InputFormat.NONE
        if string.lower(self.file_name).endswith(".dxf"):
            self.file_format = InputFormat.DXF
        elif string.lower(self.file_name).endswith(".cf2"):
            self.file_format = InputFormat.CF2
        elif string.lower(self.file_name).endswith(".plt") or \
          string.lower(self.file_name).endswith(".hpgl") or \
          string.lower(self.file_name).endswith(".hpg"):
            self.file_format = InputFormat.PLT
        elif string.lower(self.file_name).endswith(".iso"):
            self.file_format = InputFormat.ISO
        f = open("/tmp/dir.axis")
        self.open_dir = f.readline()
        f.close()
        f = open(self.file_name)
        self.source = []
        self.source = f.read()
#        self.source = string.replace(self.source, "\r", "\n")
        self.source = self.source.split("\n")
        f.close()
        return True

    def load_file(self):
        if self.file_format == InputFormat.DXF:
            self.dxf_load_file()
            self.dxf_postprocess()
            self.file_loaded = True
        elif self.file_format == InputFormat.CF2:
            self.cf2_load_file()
            self.cf2_postprocess()
            self.file_loaded = True
        elif self.file_format == InputFormat.PLT:
            self.plt_load_file()
            self.plt_postprocess()
            self.file_loaded = True
        elif self.file_format == InputFormat.ISO:
            self.iso_load_file()
            self.file_loaded = True

    def MoveTo0(self):
        if self.min.x < self.max.x:
            dx = -self.min.x
        else:
            dx = -self.max.x
        if self.min.y < self.max.y:
            dy = -self.min.y
        else:
            dy = -self.max.y
        self.min.x += dx
        self.min.y += dy
        self.max.x += dx
        self.max.y += dy
        for i in self.import_layers:
            for j in i.vectors:
                j.begin.x += dx
                j.begin.y += dy
                j.end.x += dx
                j.end.y += dy
                j.center.x += dx
                j.center.y += dy

    def Rotate(self):
        self.min.x, self.min.y = self.min.y, -self.min.x
        self.max.x, self.max.y = self.max.y, -self.max.x
        if self.min.x > self.max.x:
            self.min.x, self.max.x = self.max.x, self.min.x
        if self.min.y > self.max.y:
            self.min.y, self.max.y = self.max.y, self.min.y
        for i in self.import_layers:
            for j in i.vectors:
                j.begin.x, j.begin.y = j.begin.y, -j.begin.x
                j.end.x, j.end.y = j.end.y, -j.end.x
                j.center.x, j.center.y = j.center.y, -j.center.x
                j.view.start_angle -= 90

    def Draw(self, canvas, selected):
        canvas.delete(*self.canvas_items)
        self.canvas_items = []
        canvas.delete(*self.canvas_cross)
        self.canvas_cross = []

        t = time.time()
        minx, miny = 0.0, 0.0
        if (self.min.x < 0):
            minx = self.min.x
        if (self.min.y < 0):
            miny = self.min.y
        maxx, maxy = self.max.x, self.max.y
        if (self.max.x == 0):
            maxx = 1
        if (self.max.y == 0):
            maxy = 1

        canvasw = float(canvas.cget("width"))
        canvash = float(canvas.cget("height"))
        sx = (canvasw - PREVIEW_FRAME*2)/abs(maxx - minx)
        sy = (canvash - PREVIEW_FRAME*2)/abs(maxy - miny)
        if sx < sy:
            scale = sx
        else:
            scale = sy

        scminx = minx*scale
        scminy = miny*scale
        selected_layer = None

        for i, lay in enumerate(self.import_layers):
            if i == selected:
                selected_layer = lay
                continue
            for vec in lay.vectors:
                if vec.type == VectorType.LINE:
                    self.canvas_items.append(canvas.create_line(
                      vec.begin.x, vec.begin.y, vec.end.x, vec.end.y, fill="black"))
                    canvas.scale(self.canvas_items[-1], 0, 0, scale, -scale)
                    canvas.move(self.canvas_items[-1], PREVIEW_FRAME - scminx,
                      canvash + scminy)
                elif vec.type == VectorType.ARC:
                    x0 = vec.center.x - vec.radius
                    x1 = vec.center.x + vec.radius
                    y0 = vec.center.y - vec.radius
                    y1 = vec.center.y + vec.radius
                    self.canvas_items.append(canvas.create_arc(x0, y0, x1, y1,
                      start=vec.view.start_angle, extent=vec.view.angle, style=ARC, outline="black"))
                    canvas.scale(self.canvas_items[-1], 0, 0, scale, -scale)
                    canvas.move(self.canvas_items[-1], PREVIEW_FRAME - scminx,
                      canvash + scminy)
                elif vec.type == VectorType.TEXT:
                    self.canvas_items.append(canvas.create_text(vec.begin.x, vec.begin.y,
                      text=vec.text))
                    canvas.scale(self.canvas_items[-1], 0, 0, scale, -scale)
                    canvas.move(self.canvas_items[-1], PREVIEW_FRAME - scminx,
                      canvash + scminy)
        for vec in selected_layer.vectors:
            if vec.type == VectorType.LINE:
                self.canvas_items.append(canvas.create_line(
                  vec.begin.x, vec.begin.y, vec.end.x, vec.end.y, fill="blue"))
                canvas.scale(self.canvas_items[-1], 0, 0, scale, -scale)
                canvas.move(self.canvas_items[-1], PREVIEW_FRAME - scminx,
                  canvash + scminy)
            elif vec.type == VectorType.ARC:
                x0 = vec.center.x - vec.radius
                x1 = vec.center.x + vec.radius
                y0 = vec.center.y - vec.radius
                y1 = vec.center.y + vec.radius
                self.canvas_items.append(canvas.create_arc(x0, y0, x1, y1,
                  start=vec.view.start_angle, extent=vec.view.angle, style=ARC, outline="blue"))
                canvas.scale(self.canvas_items[-1], 0, 0, scale, -scale)
                canvas.move(self.canvas_items[-1], PREVIEW_FRAME - scminx,
                  canvash + scminy)
            elif vec.type == VectorType.TEXT:
                self.canvas_items.append(canvas.create_text(vec.begin.x, vec.begin.y,
                  text=vec.text, fill="blue"))
                canvas.scale(self.canvas_items[-1], 0, 0, scale, -scale)
                canvas.move(self.canvas_items[-1], PREVIEW_FRAME - scminx,
                  canvash + scminy)

        self.canvas_cross.append(
          canvas.create_line(-PREVIEW_FRAME, 0, PREVIEW_FRAME, 0, fill="red"))
        canvas.scale(self.canvas_cross[-1], 0, 0, scale, -scale)
        canvas.move(self.canvas_cross[-1], PREVIEW_FRAME  - scminx, canvash + scminy)
        self.canvas_cross.append(
          canvas.create_line(0, -PREVIEW_FRAME, 0, PREVIEW_FRAME, fill="red"))
        canvas.scale(self.canvas_cross[-1], 0, 0, scale, -scale)
        canvas.move(self.canvas_cross[-1], PREVIEW_FRAME  - scminx, canvash + scminy)

        print "INFO: Czas generowania podglądu: %d ms" % ((time.time() - t)*1000.0)
