#!/usr/bin/python2.4
# -*- coding: utf-8 -*-

import string
import time
from vector import *
from base_import import BaseImport

class SectionEnum:
    NONE     = 0
    HEADER   = 1
    CLASSES  = 2
    TABLES   = 3
    BLOCKS   = 4
    ENTITIES = 5
    OBJECTS  = 6
    ENDSEC   = 7

SectionName = [
    "NONE",
    "HEADER",
    "CLASSES",
    "TABLES",
    "BLOCKS",
    "ENTITIES",
    "OBJECTS",
    "ENDSEC",
    ]

class TableEntryEnum:
    NONE = 0
    APPID = 1
    BLOCK_RECORD = 2
    DIMSTYLE = 3
    LAYER = 4
    LTYPE = 5
    STYLE = 6
    UCS = 7
    VIEW = 8
    VPORT = 9

TableEntryName = [
    "NONE",
    "APPID",
    "BLOCK_RECORD",
    "DIMSTYLE",
    "LAYER",
    "LTYPE",
    "STYLE",
    "UCS",
    "VIEW",
    "VPORT",
    ]

class EntityEnum:
    NONE = 0
    LINE = 1
    ARC = 2
    CIRCLE = 3
    VERTEX = 4
    SEQEND = 5
    POLYLINE = 6
    INSERT = 7
    SPLINE = 8
    LWPOLYLINE = 9
    TEXT = 10
    MTEXT = 11
    ELLIPSE = 12
    XLINE = 13
    HATCH = 14
    SOLID = 15
    POINT = 16

EntityName = [
    "NONE",
    "LINE",
    "ARC",
    "CIRCLE",
    "VERTEX",
    "SEQEND",
    "POLYLINE",
    "INSERT",
    "SPLINE",
    "LWPOLYLINE",
    "TEXT",
    "MTEXT",
    "ELLIPSE",
    "XLINE",
    "HATCH",
    "SOLID",
    "POINT",
    ]

class HeaderVarEnum:
    NONE = 0
    ACADVER = 1
    LTSCALE = 2
    DIMSCALE = 3
    DIMALTF = 4
    MEASUREMENT = 5

HeaderVarName = [
    "NONE",
    "ACADVER",
    "LTSCALE",
    "DIMSCALE",
    "DIMALTF",
    "MEASUREMENT",
    ]

class Table:
    NONE  = 0
    LAYER = 1

class Current:
    def __init__(self):
        self.code = None
        self.data = None
        self.layer = None
        self.vector = None
        self.poly_closed = 0
        self.poly_bulge = 0
        self.poly_vertex_count = 0
        self.poly_begin = Point2d()
        self.poly_vertex = Point2d()
        self.vertex_array = []
        self.unit = 1

class DXF(BaseImport):
    def __init__(self):
        BaseImport.__init__(self)
        self.__divide_full_circle = 1
        self.__load_text = 0

    def dxf_load_file(self):
        t = time.time()
        source = self.source
        self.layers = []
        current = self.current = Current()
        self.min = Point2d(100000.0, 100000.0)
        self.max = Point2d(-100000.0, -100000.0)
        self.raw_layer = SourceLayer()

        self.var_table = []
        self.class_table = []
        self.table_table = []
        self.block_table = []
        self.ent_table = []

        self.layer_table = []

        try:
#        if True:
            self.__line = 0
            while True:
                self.__get_two_lines()
                if (current.code == 0) and (current.data == "SECTION"):
                    self.__get_two_lines()
                    if (current.code == 2):
                        try:
                            section = SectionName.index(current.data)
                        except ValueError:
                            print "WARNING: DXF: UNKNOWN MAIN SECTION: %s" % current.data
                        if section == SectionEnum.HEADER:
                            self.__parse_header()
                        elif section == SectionEnum.CLASSES:
                            self.__parse_classes()
                        elif section == SectionEnum.TABLES:
                            self.__parse_tables()
                        elif section == SectionEnum.BLOCKS:
                            self.__parse_blocks()
                        elif section == SectionEnum.ENTITIES:
                            self.__parse_entities()
                        elif section == SectionEnum.OBJECTS:
                            self.__parse_objects()
                    else:
                        break
                if (current.code == 0) and (current.data == "EOF"):
                    break

            lays = {}
            layers = []

            for lay in self.layer_table:
                layers.append(SourceLayer())
                layers[-1].name = lay[2]
                layers[-1].vectors = lays[lay[2]] = []

            if lays == {}:
                layers.append(SourceLayer())
                layers[-1].name ="LAYER"
                layers[-1].vectors = lays["LAYER"] = []

            count = {"LINE": 0, "ARC": 0, "CIRCLE": 0, "VERTEX": 0, "SEQEND": 0, "POINT": 0,
                     "POLYLINE": 0, "INSERT": 0, "SPLINE": 0, "LWPOLYLINE": 0, "SOLID": 0,
                     "TEXT": 0, "MTEXT": 0, "ELLIPSE": 0, "HATCH": 0, "UNKNOWN": 0}
            for ent in self.ent_table:
                self.__process_entity(ent, lays, count, 0, 0, 1, 1)

            index = 0
            for l in layers:
                if len(l.vectors) > 0:
                    self.layers.append(l)
                    l.config_index = index
                    index += 1
            print "DEBUG: DXF: ENTRIES: ", count

            self.layers_count = len(self.layers)
        finally:
            print "Czas przetwarzania pliku: %d ms" % ((time.time() - t)*1000.0)

    def __process_entity(self, ent, lays, count, dx, dy, sx, sy):
        current = self.current
        try:
            entity = EntityName.index(ent[0])
        except ValueError:
            entity = EntityEnum.NONE

        if entity == EntityEnum.INSERT:
            count[ent[0]] += 1
            for b in self.block_table:
                if b[2] == ent[2]:
                    block = b
                    break
#            print ent
            for blk_ent in block["VECTORS"]:
                try:
                    scalex = float(ent[41])
                except KeyError:
                    scalex = 1.0
                try:
                    scaley = float(ent[42])
                except KeyError:
                    scaley = 1.0
                self.__process_entity(blk_ent, lays, count, float(ent[10]), float(ent[20]),
                  scalex, scaley)
        elif (entity == EntityEnum.LINE) or (entity == EntityEnum.XLINE):
            count[ent[0]] += 1
            v = Vector(float(ent[10])*sx + dx, float(ent[20])*sy + dy,
              float(ent[11])*sx + dx, float(ent[21])*sy + dy, current.unit)
            v.type = VectorType.LINE
            v.config_index = 1
            self.get_line_size(v)
            try:
                lays[ent[8]].append(v)
            except KeyError:
                lays["LAYER"].append(v)
        elif entity == EntityEnum.ARC:
            count[ent[0]] += 1
            v = Vector()
            v.type = VectorType.ARC
            v.center.x = float(ent[10])*sx*current.unit + dx
            v.center.y = float(ent[20])*sy*current.unit + dy
            r = v.radius = float(ent[40])*current.unit
            v.start_angle = float(ent[50])
            v.end_angle = float(ent[51])
            v.dir = -1
            anglerad = radians(v.start_angle)
            v.begin.x = v.center.x + r*cos(anglerad)
            v.begin.y = v.center.y + r*sin(anglerad)
            anglerad = radians(v.end_angle)
            v.end.x = v.center.x + r*cos(anglerad)
            v.end.y = v.center.y + r*sin(anglerad)
            v.config_index = 1
            create_arc_view_data(v)
            self.get_arc_size(v)
            try:
                lays[ent[8]].append(v)
            except KeyError:
                lays["LAYER"].append(v)
        elif entity == EntityEnum.CIRCLE:
            count[ent[0]] += 1
            try:
                lay = lays[ent[8]]
            except KeyError:
                lay = lays["LAYER"]
            self.__process_circle(float(ent[10])*sx + dx, float(ent[20])*sy + dy,
              float(ent[40]), lay)
        elif entity == EntityEnum.ELLIPSE:
            count[ent[0]] += 1
            try:
                lay = lays[ent[8]]
            except KeyError:
                lay = lays["LAYER"]
            if (float(ent[40]) == 1.0) and (float(ent[41]) < 0.0001) and \
               (abs(float(ent[42]) - 6.2831) < 0.0001):
                cx, cy = float(ent[10])*sx + dx, float(ent[20])*sy + dy
                r = distance2d(0, 0, float(ent[11]), float(ent[21]))
                self.__process_circle(cx, cy, r, lay)
            elif (float(ent[41]) < 0.0001) and (abs(float(ent[42]) - 6.2831) < 0.0001):
                if (float(ent[11]) < 0.0001) or (float(ent[21]) < 0.0001):
                    cx, cy = float(ent[10])*sx + dx, float(ent[20])*sy + dy
                    ra = distance2d(0, 0, float(ent[11]), float(ent[21]))
                    rb = ra*float(ent[40])
                else:
                    print ent
        elif entity == EntityEnum.POLYLINE:
            count[ent[0]] += 1
            count["VERTEX"] += len(ent["VERTEX_TABLE"])
            vt = ent["VERTEX_TABLE"]
            try:
                lay = lays[ent[8]]
            except KeyError:
                lay = lays["LAYER"]
            bulge = [float(i[42]) for i in vt]
            x = [float(i[10]) for i in vt]
            y = [float(i[20]) for i in vt]
            for i in range(len(vt) - 1):
                if bulge[i] == 0:
                    v = Vector(x[i]*sx + dx, y[i]*sy + dy, x[i + 1]*sx + dx, y[i + 1]*sy + dy)
                    v.type = VectorType.LINE
                    v.config_index = 1
                    self.get_line_size(v)
                    lay.append(v)
                else:
                    self.__process_arc_vertex(x[i]*sx + dx, y[i]*sy + dy,
                      x[i + 1]*sx + dx, y[i + 1]*sy + dy, bulge[i], lay)
            if ent.has_key(70) and (int(ent[70]) & 1):
                if bulge[-1] == 0:
                    v = Vector(x[-1]*sx + dx, y[-1]*sy + dy, x[0]*sx + dx, y[0]*sy + dy)
                    v.type = VectorType.LINE
                    v.config_index = 1
                    self.get_line_size(v)
                    lay.append(v)
                else:
                    self.__process_arc_vertex(x[-1]*sx + dx, y[-1]*sy + dy,
                      x[0]*sx + dx, y[0]*sy + dy, bulge[-1], lay)
        elif entity == EntityEnum.SEQEND:
            count[ent[0]] += 1
        elif entity == EntityEnum.SPLINE:
            count[ent[0]] += 1
            pt_count = int(ent[73])
            try:
                lay = lays[ent[8]]
            except KeyError:
                lay = lays["LAYER"]
            if pt_count > 0:
                x, y = map(float, ent[10]), map(float, ent[20])
                for i in range(pt_count - 1):
                    v = Vector(x[i]*sx + dx, y[i]*sy + dy, x[i + 1]*sx + dx, y[i + 1]*sy + dy)
                    v.type = VectorType.LINE
                    v.config_index = 1
                    self.get_line_size(v)
                    lay.append(v)
                if ent.has_key(70) and (int(ent[70]) & 1):
                    v = Vector(x[-1]*sx + dx, y[-1]*sy + dy, x[0]*sx + dx, y[0]*sy + dy)
                    v.type = VectorType.LINE
                    v.config_index = 1
                    self.get_line_size(v)
                    lay.append(v)
        elif entity == EntityEnum.LWPOLYLINE:
            count[ent[0]] += 1
            x, y = map(float, ent[10]), map(float, ent[20])
            try:
                lay = lays[ent[8]]
            except KeyError:
                lay = lays["LAYER"]
            try:
                bulge = map(float, ent[42])
            except KeyError:
                bulge = [0]*len(ent[10])
            for i in range(len(ent[10]) - 1):
                if bulge[i] == 0:
                    v = Vector(x[i]*sx + dx, y[i]*sy + dy, x[i + 1]*sx + dx, y[i + 1]*sy + dy)
                    v.type = VectorType.LINE
                    v.config_index = 1
                    self.get_line_size(v)
                    lay.append(v)
                else:
                    self.__process_arc_vertex(x[-1]*sx + dx, y[-1]*sy + dy,
                      x[0]*sx + dx, y[0]*sy + dy, -bulge[i], lay)
            if ent.has_key(70) and (int(ent[70]) & 1):
                if bulge[-1] == 0:
                    v = Vector(x[-1]*sx + dx, y[-1]*sy + dy, x[0]*sx + dx, y[0]*sy + dy)
                    v.type = VectorType.LINE
                    v.config_index = 1
                    self.get_line_size(v)
                    lay.append(v)
                else:
                    self.__process_arc_vertex(x[-1]*sx + dx, y[-1]*sy + dy,
                      x[0]*sx + dx, y[0]*sy + dy, -bulge[-1], lay)
        elif entity == EntityEnum.TEXT:
            count[ent[0]] += 1
            if self.__load_text == 1:
                v = Vector()
                v.type = VectorType.TEXT
                v.begin.x = float(ent[10])*sx + dx
                v.begin.y = float(ent[20])*sy + dy
                v.text = ent[1]
                v.text_height = float(ent[40])*sy
                v.config_index = 1
                try:
                    v.text_rotate = float(ent[50])
                except KeyError:
                    v.text_rotate = 0
                try:
                    lays[ent[8]].append(v)
                except KeyError:
                    lays["LAYER"].append(v)
        elif entity == EntityEnum.MTEXT:
            count[ent[0]] += 1
            if self.__load_text == 1:
                v = Vector()
                v.type = VectorType.TEXT
                v.begin.x = float(ent[10])*sx + dx
                v.begin.y = float(ent[20])*sy + dy
                v.text = ent[1]
                v.text_height = float(ent[40])
                v.config_index = 1
                try:
                    v.text_rotate = float(ent[50])
                except KeyError:
                    v.text_rotate = 0
                try:
                    lays[ent[8]].append(v)
                except KeyError:
                    lays["LAYER"].append(v)
        elif entity == EntityEnum.HATCH:
            count[ent[0]] += 1
            x, y = map(float, ent[10]), map(float, ent[20])
#            print x
            try:
                lay = lays[ent[8]]
            except KeyError:
                lay = lays["LAYER"]
            for i in range(len(ent[10]) - 1):
                v = Vector(x[i]*sx + dx, y[i]*sy + dy, x[i + 1]*sx + dx, y[i + 1]*sy + dy)
                v.type = VectorType.LINE
                v.config_index = 1
                self.get_line_size(v)
                lay.append(v)
        elif entity == EntityEnum.SOLID:
            count[ent[0]] += 1
            try:
                lay = lays[ent[8]]
            except KeyError:
                lay = lays["LAYER"]
            v = Vector(float(ent[10])*sx + dx, float(ent[20])*sy + dy,
              float(ent[11])*sx + dx, float(ent[21])*sy + dy)
            v.type = VectorType.LINE
            v.config_index = 1
            self.get_line_size(v)
            lay.append(v)
            v = Vector(float(ent[11])*sx + dx, float(ent[21])*sy + dy,
              float(ent[12])*sx + dx, float(ent[22])*sy + dy)
            v.type = VectorType.LINE
            v.config_index = 1
            self.get_line_size(v)
            lay.append(v)
            if ent.has_key(13):
                v = Vector(float(ent[12])*sx + dx, float(ent[22])*sy + dy,
                  float(ent[13])*sx + dx, float(ent[23])*sy + dy)
                v.type = VectorType.LINE
                v.config_index = 1
                self.get_line_size(v)
                lay.append(v)
        elif entity == EntityEnum.POINT:
            count[ent[0]] += 1
            v = Vector(float(ent[10])*sx + dx, float(ent[20])*sy + dy,
              float(ent[10])*sx + dx, float(ent[20])*sy + dy)
            v.type = VectorType.LINE
            v.config_index = 1
            self.get_line_size(v)
            try:
                lays[ent[8]].append(v)
            except KeyError:
                lays["LAYER"].append(v)
        else:
            print ent
            count["UNKNOWN"] += 1

    def __process_arc_vertex(self, x0, y0, x1, y1, bulge, lay):
        v = Vector()
        v.type = VectorType.ARC
        cot = 0.5*((1.0/bulge) - bulge)
        bx = v.begin.x = x0
        by = v.begin.y = y0
        ex = v.end.x = x1
        ey = v.end.y = y1
        v.center.x = ((bx + ex) - (cot*(ey - by)))/2.0
        v.center.y = ((by + ey) + (cot*(ex - bx)))/2.0
        v.radius = distance2d(bx, by, v.center.x, v.center.y)
        if bulge > 0:
            v.dir = -1
        else:
            v.dir = 1
        v.config_index = 1
        create_arc_view_data(v)
        self.get_arc_size(v)
        lay.append(v)

    def __process_circle(self, cx, cy, r, lay):
        if self.__divide_full_circle == 1:
            v1 = Vector()
            v2 = Vector()
            v1.type = v2.type = VectorType.ARC
            v1.center.x = v2.center.x = cx
            v1.center.y = v2.center.y = v1.begin.y = v1.end.y = \
              v2.begin.y = v2.end.y = cy
            v1.radius = v2.radius = r
            v1.dir = v2.dir = -1
            v1.begin.x = v2.end.x = v1.center.x + r
            v1.end.x = v2.begin.x = v1.center.x - r
            v1.config_index = v2.config_index = 1
            lay.append(v1)
            lay.append(v2)
            create_arc_view_data(v1)
            create_arc_view_data(v2)
        else:
            v = Vector()
            v.type = VectorType.ARC
            v.center.x = cx
            v.center.y = v.begin.y = v.end.y = cy
            v.radius = r
            v.dir = -1
            v.begin.x = v.center.x + r
            v.end.x = v.center.x - r
            v.config_index = 1
            lay.append(v)
            create_arc_view_data(v)
        if (cx - r) < self.min.x:
            self.min.x = cx - r
        if (cy - r) < self.min.y:
            self.min.y = cy - r
        if (cx + r) > self.max.x:
            self.max.x = cx + r
        if (cy + r) > self.max.y:
            self.max.y = cy + r

    def __get_two_lines(self):
        try:
            self.current.code = int(self.source[self.__line])
            self.current.data = string.strip(self.source[self.__line + 1])
        except (IndexError, ValueError):
            print "ERROR: DXF: Ivalid CR/LF Format"
            raise
        except:
            print "ERROR: DXF: Error converting: ", self.source[self.__line]
            self.current.code = -1
            self.current.data = ""
            raise
        self.__line += 2

    def __parse_header(self):
        current = self.current
        scale = 1
        self.__get_two_lines()
#        print "SECTION: HEADER"
        while current.data != "ENDSEC":
            if current.code == 9:
                self.var_table.append({})
            self.var_table[-1][current.code] = current.data
            self.__get_two_lines()
        for var in self.var_table:
            try:
                name = HeaderVarName.index(var[9][1:])
                if name == HeaderVarEnum.ACADVER:
                    acad_ver = int(var[1][2:])
                    try:
                        print "DEBUG: DXF: File Version: %s" % \
                          {1003: "R9", 1006: "R10", 1009: "R11/R12", 1012: "R13", 1014: "R14",
                          1015: "AutoCAD 2000", 1018: "AutoCAD 2004"}[acad_ver]
                    except:
                        print "WARNING: DXF: Unknown File Version: %s" % var[1]
                elif name == HeaderVarEnum.LTSCALE:
                    ltscale = float(var[40])
                    print "DEBUG: DXF: LTSCALE: %.6f" % ltscale
                elif name == HeaderVarEnum.DIMSCALE:
                    dimscale = float(var[40])
                    print "DEBUG: DXF: DIMSCALE: %.6f" % dimscale
                elif name == HeaderVarEnum.DIMALTF:
                    dimaltf = float(var[40])
                    print "DEBUG: DXF: DIMALTF: %.6f" % dimaltf
                    scale *= dimaltf
                elif name == HeaderVarEnum.MEASUREMENT:
                    unit = int(var[70])
                    print "DEBUG: DXF: MEASUREMENT: %s" % ["cal", "mm"][unit]
            except ValueError:
                pass
        current.unit = scale

    def __parse_classes(self):
        current = self.current
#        print "SECTION: CLASSES"
        try:
            while current.data != "ENDSEC":
                self.__get_two_lines()
                if current.code == 0:
                    if current.data == "CLASS":
                        self.class_table.append({})
                else:
                    self.class_table[-1][current.code] = current.data
        except:
            print "WARNING: DXF: __parse_classes(): error on line %d" % self.__line

    def __parse_tables(self):
        current = self.current
#        print "SECTION: TABLES"
        while current.data != "ENDSEC":
            self.__get_two_lines()
            if (current.code == 0) and (current.data == "TABLE"):
                self.__get_two_lines()
                self.table_table.append({})
                while current.code != 0:
                    self.table_table[-1][current.code] = current.data
                    self.__get_two_lines()
#                print "    ", self.table_table[-1]
                self.table_table[-1]["TABLES"] = []
                if self.table_table[-1][2] == "LAYER":
                    self.layer_table = self.table_table[-1]["TABLES"]
                while current.data != "ENDTAB":
                    name = current.data
                    self.__get_two_lines()
                    table = {}
                    table[0] = name
                    entry = TableEntryName.index(name)
                    while current.code != 0:
                        #odczyt definicji linii
                        if (entry == TableEntryEnum.LTYPE) and (current.code == 49):
                            if table.has_key(current.code):
                                table[current.code].append(float(current.data))
                            else:
                                table[current.code] = [float(current.data)]
                        else:
                            table[current.code] = current.data
                        self.__get_two_lines()
#                    print "        ", table
                    self.table_table[-1]["TABLES"].append(table)

    def __parse_blocks(self):
        current = self.current
#        print "SECTION: BLOCKS"
        while current.data != "ENDSEC":
            self.__get_two_lines()
            if (current.code == 0) and (current.data == "BLOCK"):
                self.__get_two_lines()
                self.block_table.append({})
                while current.code != 0:
                    self.block_table[-1][current.code] = current.data
                    self.__get_two_lines()
#                print "    ", self.block_table[-1]
                self.block_table[-1]["VECTORS"] = []
                while current.data != "ENDBLK":
                    name = current.data
                    self.__get_two_lines()
                    block = {}
                    block[0] = name
                    try:
                        entity = EntityName.index(name)
                    except ValueError:
                        entity = EntityEnum.NONE

                    if entity == EntityEnum.LWPOLYLINE:
                        block[42] = {}
                        lwpoly_index = -1

                    while current.code != 0:
                        # pobieranie punktow i knot
                        if (entity == EntityEnum.SPLINE) and (current.code in [10, 11, 20, 21, 30, 31, 40]):
                            if block.has_key(current.code):
                                block[current.code].append(float(current.data))
                            else:
                                block[current.code] = [float(current.data)]
                        # pobieranie punktow i bulge
                        elif entity == EntityEnum.LWPOLYLINE:
                            if current.code == 10:
                                lwpoly_index += 1
                                if block.has_key(current.code):
                                    block[10].append(float(current.data))
                                else:
                                    block[10] = [float(current.data)]
                            elif current.code == 20:
                                if block.has_key(current.code):
                                    block[20].append(float(current.data))
                                else:
                                    block[20] = [float(current.data)]
                            elif current.code == 42:
                                block[42][lwpoly_index] = float(current.data)
                            else:
                                block[current.code] = current.data
                        elif (entity == EntityEnum.HATCH) and (current.code in [10, 20]):
                            if block.has_key(current.code):
                                block[current.code].append(float(current.data))
                            else:
                                block[current.code] = [float(current.data)]
                        else:
                            block[current.code] = current.data
                        self.__get_two_lines()
#                    print "        ", block
                    if entity == EntityEnum.POLYLINE:
                        vertex_table = block["VERTEX_TABLE"] = []
                        self.block_table[-1]["VECTORS"].append(block)
                    elif entity == EntityEnum.LWPOLYLINE:
                        try:
                            dic = block[42]
                            block[42] = [0]*len(block[10])
                            for k, v in dic.iteritems():
                                block[42][k] = v
                        except KeyError:
                            block[42] = [0]*len(block[10])
                        self.block_table[-1]["VECTORS"].append(block)
                    elif entity == EntityEnum.VERTEX:
                        if not block.has_key(42):
                            block[42] = 0.0
                        vertex_table.append(block)
                    else:
                        self.block_table[-1]["VECTORS"].append(block)

    def __parse_entities(self):
        current = self.current
#        print "SECTION: ENTITIES"
        ent_table = []
        self.__get_two_lines()
        while current.data != "ENDSEC":
            name = current.data
            self.__get_two_lines()
            ent = {}
            ent[0] = name
            try:
                entity = EntityName.index(name)
            except ValueError:
                entity = EntityEnum.NONE

            if entity == EntityEnum.LWPOLYLINE:
                ent[42] = {}
                lwpoly_index = -1

            while current.code != 0:
                # pobieranie punktow i knot
                if (entity == EntityEnum.SPLINE) and (current.code in [10, 11, 20, 21, 30, 31, 40]):
                    if ent.has_key(current.code):
                        ent[current.code].append(float(current.data))
                    else:
                        ent[current.code] = [float(current.data)]
                # pobieranie punktow i bulge
                elif entity == EntityEnum.LWPOLYLINE:
                    if current.code == 10:
                        lwpoly_index += 1
                        if ent.has_key(current.code):
                            ent[10].append(float(current.data))
                        else:
                            ent[10] = [float(current.data)]
                    elif current.code == 20:
                        if ent.has_key(current.code):
                            ent[20].append(float(current.data))
                        else:
                            ent[20] = [float(current.data)]
                    elif current.code == 42:
                        ent[42][lwpoly_index] = float(current.data)
                    else:
                        ent[current.code] = current.data
                elif entity == EntityEnum.HATCH:
                    if current.code == 92:
                        ent[10] = []
                        ent[20] = []
                        ent[current.code] = current.data
                    elif current.code in [10, 20]:
                        if (not ent.has_key(98)) and ent.has_key(92):
                            ent[current.code].append(float(current.data))
                    else:
                        ent[current.code] = current.data
                else:
                    ent[current.code] = current.data
                self.__get_two_lines()
#            print "        ", ent
            if entity == EntityEnum.POLYLINE:
                vertex_table = ent["VERTEX_TABLE"] = []
                ent_table.append(ent)
            elif entity == EntityEnum.LWPOLYLINE:
                try:
                    dic = ent[42]
                    ent[42] = [0]*len(ent[10])
                    for k, v in dic.iteritems():
                        ent[42][k] = v
                except KeyError:
                    ent[42] = [0]*len(ent[10])
                ent_table.append(ent)
            elif entity == EntityEnum.VERTEX:
                # jesli nie ma bulge ustaw wartosc na 0
                if not ent.has_key(42):
                    ent[42] = 0.0
                vertex_table.append(ent)
            elif entity == EntityEnum.INSERT:
                if ent.has_key(66):
                    print "INSERT: ATTRIBUTES FOLLOW", ent[66]
            elif entity == EntityEnum.HATCH:
                print ent[10]
            else:
                ent_table.append(ent)
        self.ent_table = ent_table

    def __parse_objects(self):
        current = self.current
        self.__get_two_lines()
#        print "SECTION: OBJECTS"
        while current.data != "ENDSEC":
            self.__get_two_lines()
