#!/usr/bin/python2.4
# -*- coding: utf-8 -*-

import time
from vector import *
from base_import import *

class LevelTypeEnum:
    DOWN = -1
    ZERO = 0
    UP = 1

class VelTypeEnum:
    NORMAL = 0
    DRILL = 1

class SpindleParams:
    def __init__(self):
        self.mill_mode = -1
        self.enable_gcode = ""
        self.enable_delay_gcode = ""
        self.disable_gcode = ""
        self.disable_delay_gcode = ""

class OverrideParams:
    def __init__(self):
        self.control = -1
        self.enable_gcode = ""
        self.disable_gcode = ""

class RapidParams:
    def __init__(self):
        self.control = -1
        self.gcode = ""

class ToolParams:
    def __init__(self):
        self.control = -1

class ScanParams:
    def __init__(self):
        self.xsize = 0.0
        self.ysize = 0.0
        self.xstep = 0.0
        self.ystep = 0.0

class GrindParams:
    def __init__(self):
        self.diameter = 0.0
        self.sizex = 0.0
        self.sizey = 0.0
        self.up = 0.0
        self.down = 0.0
        self.dirx = 0
        self.diry = 0

class ConfigsParams:
    def __init__(self):
        self.change_gcode = None

class ManualParams:
    def __init__(self):
        self.up = 5

class GeneratorParams:
    def __init__(self):
        self.axis_count = 0
        self.axis_names = ""
        self.out_precision_fields = 0
        self.precision = 0
        self.spindle = SpindleParams()
        self.override = OverrideParams()
        self.rapid_delay = RapidParams()
        self.configs = ConfigsParams()

        self.manual = ManualParams()

        self._base_velocity = 0
        self._tool = ToolParams()
        self._scan = ScanParams()
        self._grind = GrindParams()
        self._gcode_type = -1
        self._layer_index = -1
        self._delete_small = -1
        self._local_config = -1
        self._fix_inverted_vectors = -1
        self._one_layer = -1
        self._dir_sort = -1

class GeneratorState:
    def __init__(self):
        self.reset()

    def reset(self):
        self.cx = -1
        self.cy = -1
        self.cz = -1
        self.config_index = -1
        self.tool = -1
        self.spindle = 0
        self.override = 1
        self.comp_enable = 0
        self.level = LevelTypeEnum.DOWN
        self.z_level = 0
        self.down_step = 0
        self.global_curr_pass = 0
        self.local_curr_pass = 0
        self.g0_format = ""
        self.g1_format = ""
        self.g2_format = ""
        self.g3_format = ""
        self.vec_count = -1
        self.dir = 0
        self.diameter = 0
        self.path_width = 0
        self.out = []

class Gen:
    def __init__(self):
        self.signals = []
        self.overrides = []
        self.params = GeneratorParams()

    def __idle_func(self, arg1=None, arg2=None):
        return False

    def __dir_sort_func(self, vec):
        if vec.type != VectorType.LINE:
            return
        if (abs(vec.end.x - vec.begin.x) < self.params.precision) and (vec.end.y < vec.begin.y):
            vec.begin.x, vec.end.x = vec.end.x, vec.begin.x
            vec.begin.y, vec.end.y = vec.end.y, vec.begin.y
        elif (vec.end.x < vec.begin.x) and (vec.end.y < vec.begin.y):
            vec.begin.x, vec.end.x = vec.end.x, vec.begin.x
            vec.begin.y, vec.end.y = vec.end.y, vec.begin.y
        elif (vec.end.x < vec.begin.x) and (abs(vec.end.y - vec.begin.y) < self.params.precision):
            vec.begin.x, vec.end.x = vec.end.x, vec.begin.x
            vec.begin.y, vec.end.y = vec.end.y, vec.begin.y
        elif (vec.end.x < vec.begin.x) and (vec.end.y > vec.begin.y):
            vec.begin.x, vec.end.x = vec.end.x, vec.begin.x
            vec.begin.y, vec.end.y = vec.end.y, vec.begin.y

    def __fix_inverted_vectors(self, n, lay, vec):
        if self.params._fix_inverted_vectors == 1:
            try:
                next = lay.vectors[n + 1]
            except IndexError:
                next = lay.vectors[0]
            p = self.params
            if (n < self.gen.vec_count - 1) and \
              (abs(vec.begin.x - next.begin.x) <= p.precision) and \
              (abs(vec.begin.y - next.begin.y) <= p.precision):
                vec.end.x, vec.begin.x = vec.begin.x, vec.end.x
                vec.end.y, vec.begin.y = vec.begin.y, vec.end.y
                vec.dir = -vec.dir
            elif (n < self.gen.vec_count - 1) and \
              (abs(vec.begin.x - next.end.x) <= p.precision) and \
              (abs(vec.begin.y - next.end.y) <= p.precision):
                next.end.x, next.begin.x = next.begin.x, next.end.x
                next.end.y, next.begin.y = next.begin.y, next.end.y
                next.dir = -next.dir
                vec.end.x, vec.begin.x = vec.begin.x, vec.end.x
                vec.end.y, vec.begin.y = vec.begin.y, vec.end.y
                vec.dir = -vec.dir
            elif (abs(vec.end.x - self.gen.cx) <= p.precision) and \
              (abs(vec.end.y - self.gen.cy) <= p.precision):
                vec.end.x, vec.begin.x = vec.begin.x, vec.end.x
                vec.end.y, vec.begin.y = vec.begin.y, vec.end.y
                vec.dir = -vec.dir

    def __spindle_enable(self):
        if self.gen.spindle == 0:
            self.gen.out.append(self.params.spindle.enable_gcode)
            self.gen.out.append(self.params.spindle.enable_delay_gcode)
            self.gen.spindle = 1

    def __spindle_disable(self):
        if self.gen.spindle == 1:
            self.gen.out.append(self.params.spindle.disable_gcode)
            self.gen.out.append(self.params.spindle.disable_delay_gcode)
            self.gen.spindle = 0

    def __override_enable(self):
        if self.gen.override == 0:
            self.gen.out.append(self.params.override.enable_gcode)
            self.gen.override = 1

    def __override_disable(self):
        if self.gen.override == 1:
            self.gen.out.append(self.params.override.disable_gcode)
            self.gen.override = 0

    def __change_config(self, index):
        if self.gen.config_index != index:
            self.gen.out.append(self.gen.change_config_format % index)
            self.gen.config_index = index

    def __rapid_delay_func(self):
        self.gen.out.append(self.params.rapid_delay.gcode)

    def __move_up(self, vec, coords):
        if self.gen.level != LevelTypeEnum.UP:
            self.gen.out.append(self.gen.g0_vert_format % self.gen.up)
            coords[len(self.gen.out) - 1] = [0, self.gen.config_index,
              (self.gen.cx, self.gen.cy, self.gen.z_level), (self.gen.cx, self.gen.cy, self.gen.up)]
            self.gen.level = LevelTypeEnum.UP

    def __move_zero(self, vec, coords):
        if self.gen.level != LevelTypeEnum.ZERO:
            self.gen.out.append(self.gen.g0_vert_zero_format % self.gen.zero)
            coords[len(self.gen.out) - 1] = [0, self.gen.config_index,
              (self.gen.cx, self.gen.cy, self.gen.up), (self.gen.cx, self.gen.cy, self.gen.zero)]
            self.gen.level = LevelTypeEnum.ZERO

    def __move_down(self, vec, coords):
        if self.gen.level != LevelTypeEnum.DOWN:
            self.gen.out.append(self.gen.g1_vert_format % self.gen.z_level)
            coords[len(self.gen.out) - 1] = [1, self.gen.config_index,
              (self.gen.cx, self.gen.cy, self.gen.zero), (self.gen.cx, self.gen.cy, self.gen.z_level)]
            self.gen.level = LevelTypeEnum.DOWN

    def __next_pass_func(self):
        self.gen.z_level -= self.gen.down_step
        self.gen.level = LevelTypeEnum.ZERO

    def __local_restore_pass_func(self):
        self.gen.z_level = -self.gen.down_step
        self.gen.level = LevelTypeEnum.DOWN

    def __update_curr_beg_pos(self, vec):
        self.gen.cx, self.gen.cy = vec.begin.x, vec.begin.y

    def __update_curr_end_pos(self, vec):
        self.gen.cx, self.gen.cy = vec.end.x, vec.end.y

    def __limit_size_func(self, vec):
        if (vec.type == VectorType.LINE) and (hypot(vec.end.x - vec.begin.x, vec.end.y - vec.begin.y) < self.params.precision):
            return True
        else:
            return False

    def __generate_rapid2d(self, vec, coords):
        self.gen.out.append(self.gen.g0_format % (vec.begin.x, vec.begin.y))
        coords[len(self.gen.out) - 1] = [0, self.gen.config_index, (self.gen.cx, self.gen.cy), (vec.begin.x, vec.begin.y)]

    def __generate_rapid3d(self, vec, coords):
        self.gen.out.append(self.gen.g0_format % (vec.begin.x, vec.begin.y, self.gen.up))
        coords[len(self.gen.out) - 1] = [0, self.gen.config_index, (self.gen.cx, self.gen.cy, self.gen.up), (vec.begin.x, vec.begin.y, self.gen.up)]

    def __generate_line2d(self, vec, coords):
        self.gen.out.append(self.gen.g1_format % (vec.end.x, vec.end.y))
        coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (vec.begin.x, vec.begin.y), (vec.end.x, vec.end.y)]

    def __generate_line3d(self, vec, coords):
        self.gen.out.append(self.gen.g1_format % (vec.end.x, vec.end.y, self.gen.z_level))
        coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (vec.begin.x, vec.begin.y, self.gen.z_level), (vec.end.x, vec.end.y, self.gen.z_level)]

    def __generate_corr_line2d(self, vec, coords):
        self.gen.out.append(self.gen.g1_format % (vec.begin.x, vec.begin.y))
        coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (self.gen.cx, self.gen.cy), (vec.begin.x, vec.begin.y)]

    def __generate_corr_line3d(self, vec, coords):
        self.gen.out.append(self.gen.g1_format % (vec.begin.x, vec.begin.y, self.gen.z_level))
        coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (self.gen.cx, self.gen.cy, self.gen.z_level), (vec.begin.x, vec.begin.y, self.gen.z_level)]

    def __generate_line_multipass2d(self, vec, coords):
        x1 = vec.begin.x
        y1 = vec.begin.y
        x2 = vec.end.x
        y2 = vec.end.y
        dx = x2 - x1
        dy = y2 - y1
        if hypot(dx, dy) < 0.05:
            self.gen.out.append(self.gen.g1_format % (vec.end.x, vec.end.y))
            coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (vec.begin.x, vec.begin.y), (vec.end.x, vec.end.y)]
        else:
            corr = (self.gen.diameter - self.gen.path_width)/2.0
            angle = find_angle(x1 + 1, y1, x1, y1, x2, y2)
            if self.gen.dir == 0:
                xend1 = corr*cos(radians(angle - 90)) + x1
                yend1 = corr*sin(radians(angle - 90)) + y1
                xbeg1 = corr*cos(radians(angle + 90)) + x1
                ybeg1 = corr*sin(radians(angle + 90)) + y1
            else:
                xend1 = corr*cos(radians(angle + 90)) + x1
                yend1 = corr*sin(radians(angle + 90)) + y1
                xbeg1 = corr*cos(radians(angle - 90)) + x1
                ybeg1 = corr*sin(radians(angle - 90)) + y1
            xend2 = xend1 + dx
            yend2 = yend1 + dy
            xbeg2 = xbeg1 + dx
            ybeg2 = ybeg1 + dy
            self.gen.out.append(self.gen.g1_format % (xend1, yend1))
            coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (x1, y1), (xend1, yend1)]
            self.gen.out.append(self.gen.g1_format % (xend2, yend2))
            coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (xend1, yend1), (xend2, yend2)]
            self.gen.out.append(self.gen.g1_format % (xbeg2, ybeg2))
            coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (xend2, yend2), (xbeg2, ybeg2)]
            self.gen.out.append(self.gen.g1_format % (xbeg1, ybeg1))
            coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (xbeg2, ybeg2), (xbeg1, ybeg1)]
            self.gen.out.append(self.gen.g1_format % (x1, y1))
            coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (xbeg1, ybeg1), (x1, y1)]

    def __generate_line_multipass3d(self, vec, coords):
        x1 = vec.begin.x
        y1 = vec.begin.y
        x2 = vec.end.x
        y2 = vec.end.y
        dx = x2 - x1
        dy = y2 - y1
        if hypot(dx, dy) < 0.05:
            self.gen.out.append(self.gen.g1_format % (vec.end.x, vec.end.y, self.gen.z_level))
            coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (vec.begin.x, vec.begin.y, self.gen.z_level), (vec.end.x, vec.end.y, self.gen.z_level)]
        else:
            corr = (self.gen.diameter - self.gen.path_width)/2.0
            angle = find_angle(x1 + 1, y1, x1, y1, x2, y2)
            if self.gen.dir == 0:
                xend1 = corr*cos(radians(angle - 90)) + x1
                yend1 = corr*sin(radians(angle - 90)) + y1
                xbeg1 = corr*cos(radians(angle + 90)) + x1
                ybeg1 = corr*sin(radians(angle + 90)) + y1
            else:
                xend1 = corr*cos(radians(angle + 90)) + x1
                yend1 = corr*sin(radians(angle + 90)) + y1
                xbeg1 = corr*cos(radians(angle - 90)) + x1
                ybeg1 = corr*sin(radians(angle - 90)) + y1
            xend2 = xend1 + dx
            yend2 = yend1 + dy
            xbeg2 = xbeg1 + dx
            ybeg2 = ybeg1 + dy
            self.gen.out.append(self.gen.g1_format % (xend1, yend1, self.gen.z_level))
            coords[len(self.gen.out) - 1] = [1, self.gen.config_index,
              (x1, y1, self.gen.z_level), (xend1, yend1, self.gen.z_level)]
            self.gen.out.append(self.gen.g1_format % (xend2, yend2, self.gen.z_level))
            coords[len(self.gen.out) - 1] = [1, self.gen.config_index,
              (xend1, yend1, self.gen.z_level), (xend2, yend2, self.gen.z_level)]
            self.gen.out.append(self.gen.g1_format % (xbeg2, ybeg2, self.gen.z_level))
            coords[len(self.gen.out) - 1] = [1, self.gen.config_index,
              (xend2, yend2, self.gen.z_level), (xbeg2, ybeg2, self.gen.z_level)]
            self.gen.out.append(self.gen.g1_format % (xbeg1, ybeg1, self.gen.z_level))
            coords[len(self.gen.out) - 1] = [1, self.gen.config_index,
              (xbeg2, ybeg2, self.gen.z_level), (xbeg1, ybeg1, self.gen.z_level)]
            self.gen.out.append(self.gen.g1_format % (x1, y1, self.gen.z_level))
            coords[len(self.gen.out) - 1] = [1, self.gen.config_index,
              (xbeg1, ybeg1, self.gen.z_level), (x1, y1, self.gen.z_level)]

    def __generate_arc2d(self, vec, coords):
        if vec.dir == -1:
            self.gen.out.append(self.gen.g3_format % (vec.end.x, vec.end.y, vec.center.x - vec.begin.x, vec.center.y - vec.begin.y))
            coords[len(self.gen.out) - 1] = [3, self.gen.config_index, (vec.begin.x, vec.begin.y), (vec.end.x, vec.end.y), (vec.center.x, vec.center.y)]
        elif vec.dir == 1:
            self.gen.out.append(self.gen.g2_format % (vec.end.x, vec.end.y, vec.center.x - vec.begin.x, vec.center.y - vec.begin.y))
            coords[len(self.gen.out) - 1] = [2, self.gen.config_index, (vec.begin.x, vec.begin.y), (vec.end.x, vec.end.y), (vec.center.x, vec.center.y)]

    def __generate_arc3d(self, vec, coords):
        if vec.dir == -1:
            self.gen.out.append(self.gen.g3_format % (vec.end.x, vec.end.y, vec.center.x - vec.begin.x, vec.center.y - vec.begin.y))
            coords[len(self.gen.out) - 1] = [3, self.gen.config_index, (vec.begin.x, vec.begin.y, self.gen.z_level), (vec.end.x, vec.end.y, self.gen.z_level), (vec.center.x, vec.center.y, self.gen.z_level)]
        elif vec.dir == 1:
            self.gen.out.append(self.gen.g2_format % (vec.end.x, vec.end.y, vec.center.x - vec.begin.x, vec.center.y - vec.begin.y))
            coords[len(self.gen.out) - 1] = [2, self.gen.config_index, (vec.begin.x, vec.begin.y, self.gen.z_level), (vec.end.x, vec.end.y, self.gen.z_level), (vec.center.x, vec.center.y, self.gen.z_level)]

    def __generate_arc_multipass2d(self, vec, coords):
        corr = (self.gen.path_width - self.gen.diameter)/2.0
        x1 = vec.begin.x
        y1 = vec.begin.y
        x2 = vec.end.x
        y2 = vec.end.y
        cx = vec.center.x
        cy = vec.center.y
        alfa = find_angle(cx + 1, cy, cx, cy, x1, y1)
        beta = find_angle(cx + 1, cy, cx, cy, x2, y2)
        sin_alfa = sin(radians(alfa))
        cos_alfa = cos(radians(alfa))
        sin_beta = sin(radians(beta))
        cos_beta = cos(radians(beta))
        if self.gen.dir == 0:
            xleft_beg = -corr*cos_alfa + x1
            yleft_beg = -corr*sin_alfa + y1
            xright_beg = corr*cos_alfa + x1
            yright_beg = corr*sin_alfa + y1
            xleft_end = -corr*cos_beta + x2
            yleft_end = -corr*sin_beta + y2
            xright_end = corr*cos_beta + x2
            yright_end = corr*sin_beta + y2
        else:
            xleft_beg = corr*cos_alfa + x1
            yleft_beg = corr*sin_alfa + y1
            xright_beg = -corr*cos_alfa + x1
            yright_beg = -corr*sin_alfa + y1
            xleft_end = corr*cos_beta + x2
            yleft_end = corr*sin_beta + y2
            xright_end = -corr*cos_beta + x2
            yright_end = -corr*sin_beta + y2
        if vec.dir == -1:
            arc_codes = [self.gen.g3_format, self.gen.g2_format, 3, 2]
        elif vec.dir == 1:
            arc_codes = [self.gen.g2_format, self.gen.g3_format, 2, 3]
        self.gen.out.append(self.gen.g1_format % (xleft_beg, yleft_beg))
        coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (x1, y1), (xleft_beg, yleft_beg)]

        self.gen.out.append(arc_codes[0] % (xleft_end, yleft_end, cx - xleft_beg, cy - yleft_beg))
        coords[len(self.gen.out) - 1] = [arc_codes[2], self.gen.config_index, (xleft_beg, yleft_beg), (xleft_end, yleft_end), (cx, cy)]

        self.gen.out.append(self.gen.g1_format % (xright_end, yright_end))
        coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (xleft_end, yleft_end), (xright_end, yright_end)]

        self.gen.out.append(arc_codes[1] % (xright_beg, yright_beg, cx - xright_end, cy - yright_end))
        coords[len(self.gen.out) - 1] = [arc_codes[3], self.gen.config_index, (xright_end, yright_end), (xright_beg, yright_beg), (cx, cy)]

        self.gen.out.append(self.gen.g1_format % (x1, y1))
        coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (xright_beg, yright_beg), (x1, y1)]

    def __generate_arc_multipass3d(self, vec, coords):
        corr = (self.gen.path_width - self.gen.diameter)/2.0
        x1 = vec.begin.x
        y1 = vec.begin.y
        x2 = vec.end.x
        y2 = vec.end.y
        cx = vec.center.x
        cy = vec.center.y
        alfa = find_angle(cx + 1, cy, cx, cy, x1, y1)
        beta = find_angle(cx + 1, cy, cx, cy, x2, y2)
        sin_alfa = sin(radians(alfa))
        cos_alfa = cos(radians(alfa))
        sin_beta = sin(radians(beta))
        cos_beta = cos(radians(beta))
        if self.gen.dir == 0:
            xleft_beg = -corr*cos_alfa + x1
            yleft_beg = -corr*sin_alfa + y1
            xright_beg = corr*cos_alfa + x1
            yright_beg = corr*sin_alfa + y1
            xleft_end = -corr*cos_beta + x2
            yleft_end = -corr*sin_beta + y2
            xright_end = corr*cos_beta + x2
            yright_end = corr*sin_beta + y2
        else:
            xleft_beg = corr*cos_alfa + x1
            yleft_beg = corr*sin_alfa + y1
            xright_beg = -corr*cos_alfa + x1
            yright_beg = -corr*sin_alfa + y1
            xleft_end = corr*cos_beta + x2
            yleft_end = corr*sin_beta + y2
            xright_end = -corr*cos_beta + x2
            yright_end = -corr*sin_beta + y2
        if vec.dir == -1:
            arc_codes = [self.gen.g3_format, self.gen.g2_format, 3, 2]
        elif vec.dir == 1:
            arc_codes = [self.gen.g2_format, self.gen.g3_format, 2, 3]
        self.gen.out.append(self.gen.g1_format % (xleft_beg, yleft_beg, self.gen.z_level))
        coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (x1, y1, self.gen.z_level), (xleft_beg, yleft_beg, self.gen.z_level)]

        self.gen.out.append(arc_codes[0] % (xleft_end, yleft_end, cx - xleft_beg, cy - yleft_beg))
        coords[len(self.gen.out) - 1] = [arc_codes[2], self.gen.config_index, (xleft_beg, yleft_beg, self.gen.z_level), (xleft_end, yleft_end, self.gen.z_level), (cx, cy, self.gen.z_level)]

        self.gen.out.append(self.gen.g1_format % (xright_end, yright_end, self.gen.z_level))
        coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (xleft_end, yleft_end, self.gen.z_level), (xright_end, yright_end, self.gen.z_level)]

        self.gen.out.append(arc_codes[1] % (xright_beg, yright_beg, cx - xright_end, cy - yright_end))
        coords[len(self.gen.out) - 1] = [arc_codes[3], self.gen.config_index, (xright_end, yright_end, self.gen.z_level), (xright_beg, yright_beg, self.gen.z_level), (cx, cy, self.gen.z_level)]

        self.gen.out.append(self.gen.g1_format % (x1, y1, self.gen.z_level))
        coords[len(self.gen.out) - 1] = [1, self.gen.config_index, (xright_beg, yright_beg, self.gen.z_level), (x1, y1, self.gen.z_level)]

    def __sort_vectors(self, vectors):
        return vectors

    def generate_gcode(self, lay_num):
        t = time.time()
        p = self.params
        gen = self.gen = GeneratorState()
        gen.reset()

        gen.change_config_format = "%s P%%d" % p.configs.change_gcode

        if p._delete_small == 1:
            limit_size_func = self.__limit_size_func
        else:
            limit_size_func = self.__idle_func

        if p.spindle.mill_mode == 0:
            spindle_enable_func = self.__spindle_enable
            spindle_disable_func = self.__spindle_disable
        else:
            spindle_enable_func = self.__idle_func
            spindle_disable_func = self.__idle_func

        if p.override.control == 1:
            override_enable_func = self.__override_enable
            override_disable_func = self.__override_disable
        else:
            override_enable_func = self.__idle_func
            override_disable_func = self.__idle_func

        if p.rapid_delay.control == 1:
            rapid_delay_func = self.__rapid_delay_func
        else:
            rapid_delay_func = self.__idle_func

        if p.axis_count == 3:
            n = p.out_precision_fields
            axis0 = p.axis_names[0]
            axis1 = p.axis_names[1]
            axis2 = p.axis_names[2]
            gen.g0_format = "G0 %s%%.%df %s%%.%df %s%%.%df" % (axis0, n, axis1, n, axis2, n)
            gen.g0_vert_format = "G0 %s%%.%df" % (axis2, n)
            gen.g0_vert_zero_format = "G0 %s%%.%df" % (axis2, n)
            gen.g1_format = "G1 %s%%.%df %s%%.%df %s%%.%df" % (axis0, n, axis1, n, axis2, n)
            gen.g1_vert_format = "G1 %s%%.%df" % (axis2, n)
            gen.g2_format = "G2 %s%%.%df %s%%.%df I%%.%df J%%.%df" % (axis0, n, axis1, n, n, n)
            gen.g3_format = "G3 %s%%.%df %s%%.%df I%%.%df J%%.%df" % (axis0, n, axis1, n, n, n)

            move_up_func = self.__move_up
            move_zero_func = self.__move_zero
            move_down_func = self.__move_down
            generate_rapid_func = self.__generate_rapid3d
        elif p.axis_count == 2:
            n = p.out_precision_fields
            axis0 = p.axis_names[0]
            axis1 = p.axis_names[1]
            gen.g0_format = "G0 %s%%.%df %s%%.%df" % (axis0, n, axis1, n)
            gen.g1_format = "G1 %s%%.%df %s%%.%df" % (axis0, n, axis1, n)
            gen.g2_format = "G2 %s%%.%df %s%%.%df I%%.%df J%%.%df" % (axis0, n, axis1, n, n, n)
            gen.g3_format = "G3 %s%%.%df %s%%.%df I%%.%df J%%.%df" % (axis0, n, axis1, n, n, n)

            move_up_func = self.__idle_func
            move_zero_func = self.__idle_func
            move_down_func = self.__idle_func
            generate_rapid_func = self.__generate_rapid2d

        if p._layer_index == -1:
            layers_list = self.layers
        else:
            layers_list = [self.layers[p._layer_index]]

        if p._local_config == 1:
            layers_list = [self.single_layer]

        if p._dir_sort == 1:
            dir_sort_func = self.__dir_sort_func
        else:
            dir_sort_func = self.__idle_func

        coords = {}
        gen.out = []
        gen.out.append("S%d" % p._spindle_velocity)
        gen.out.append("F%d" % p._base_velocity)
#        gen.out.append("#PATH_MODE")

        for nr, lay in enumerate(layers_list):
            if nr != lay_num:
                continue
# konfigurowanie lokalnych parametrow warstwy
            if p.axis_count == 3:
                gen.up = lay.params[0]
                gen.zero = lay.params[1]
                if lay.params[3] > 1:
                    gen.down_step = float(lay.params[2])/float(lay.params[3])
                    gen.z_level = -gen.down_step
                    if lay.multipass_type == MultipassTypeEnum.GLOBAL:
                        global_pass_count = int(lay.params[3])
                        local_pass_count = 1
                        global_next_pass_func = self.__next_pass_func
                        local_next_pass_func = self.__idle_func
                        local_restore_pass_func = self.__idle_func
                        if (lay.width_comp == 1) and (lay.diameter != lay.path_width):
                            update_curr_pos_func = self.__update_curr_beg_pos
                        else:
                            update_curr_pos_func = self.__update_curr_end_pos
                    elif lay.multipass_type == MultipassTypeEnum.LOCAL:
                        global_pass_count = 1
                        local_pass_count = int(lay.params[3])
                        global_next_pass_func = self.__idle_func
                        local_next_pass_func = self.__next_pass_func
                        local_restore_pass_func = self.__local_restore_pass_func
                        update_curr_pos_func = self.__update_curr_end_pos
                        if (lay.width_comp == 1) and (lay.diameter != lay.path_width):
                            update_curr_pos_func = self.__update_curr_end_pos
                        else:
                            update_curr_pos_func = self.__update_curr_end_pos
                else:
                    gen.down_step = lay.params[2]
                    gen.z_level = -gen.down_step
                    global_pass_count = 1
                    local_pass_count = 1
                    global_next_pass_func = self.__idle_func
                    local_next_pass_func = self.__idle_func
                    local_restore_pass_func = self.__idle_func
                    if lay.multipass_type == MultipassTypeEnum.GLOBAL:
                        if (lay.width_comp == 1) and (lay.diameter != lay.path_width):
                            update_curr_pos_func = self.__update_curr_beg_pos
                        else:
                            update_curr_pos_func = self.__update_curr_end_pos
                    elif lay.multipass_type == MultipassTypeEnum.LOCAL:
                        if (lay.width_comp == 1) and (lay.diameter != lay.path_width):
                            update_curr_pos_func = self.__update_curr_beg_pos
                        else:
                            update_curr_pos_func = self.__update_curr_end_pos
                if lay.sort == 1:
                    vectors = []
                    vectors.extend(self.__sort_vectors(lay.vectors))
                else:
                    vectors = []
                    vectors.extend(lay.vectors)
                if (lay.width_comp == 1) and (lay.diameter != lay.path_width):
                    generate_line_func = self.__generate_line_multipass3d
                    generate_arc_func = self.__generate_arc_multipass3d
                    gen.dir = lay.dir
                    gen.diameter = lay.diameter
                    gen.path_width = lay.path_width
                else:
                    generate_line_func = self.__generate_line3d
                    generate_corr_line_func = self.__generate_corr_line3d
                    generate_arc_func = self.__generate_arc3d
                if p._local_config == 1:
                    local_change_config_func = self.__change_config
                    if p._tool.changer == 1:
                        local_change_tool_func = self.__change_tool
                    else:
                        local_change_tool_func = self.__idle_func
                    global_change_config_func = self.__idle_func
                    global_change_tool_func = self.__idle_func
                else:
                    global_change_config_func = self.__change_config
                    if p._tool.changer == 1:
                        global_change_tool_func = self.__change_tool
                    else:
                        global_change_tool_func = self.__idle_func
                    local_change_config_func = self.__idle_func
                    local_change_tool_func = self.__idle_func
            elif p.axis_count == 2:
                global_pass_count = 1
                local_pass_count = 1
                global_next_pass_func = self.__idle_func
                local_next_pass_func = self.__idle_func
                local_restore_pass_func = self.__idle_func
                if (lay.width_comp == 1) and (lay.diameter != lay.path_width):
                    generate_line_func = self.__generate_line_multipass2d
                    generate_arc_func = self.__generate_arc_multipass2d
                    update_curr_pos_func = self.__update_curr_beg_pos
                    gen.dir = lay.dir
                    gen.diameter = lay.diameter
                    gen.path_width = lay.path_width
                else:
                    generate_line_func = self.__generate_line2d
                    generate_corr_line_func = self.__generate_corr_line2d
                    generate_arc_func = self.__generate_arc2d
                    update_curr_pos_func = self.__update_curr_end_pos
                if lay.sort == 1:
                    vectors = []
                    vectors.extend(self.__sort_vectors(lay.vectors))
                else:
                    vectors = []
                    vectors.extend(lay.vectors)
                if p._local_config == 1:
                    local_change_config_func = self.__change_config
                    if p._tool.changer == 1:
                        local_change_tool_func = self.__change_tool
                    else:
                        local_change_tool_func = self.__idle_func
                    global_change_config_func = self.__idle_func
                    global_change_tool_func = self.__idle_func
                else:
                    global_change_config_func = self.__change_config
                    if p._tool.changer == 1:
                        global_change_tool_func = self.__change_tool
                    else:
                        global_change_tool_func = self.__idle_func
                    local_change_config_func = self.__idle_func
                    local_change_tool_func = self.__idle_func
# generowanie gcodu z biezacej warstwy z aktualnymi parametrami
            for self.gen.global_curr_pass in range(global_pass_count):
 #               global_change_config_func(lay.config_index)
  #              global_change_tool_func(lay.tool)
                for n, vec in enumerate(vectors):
                    if limit_size_func(vec):
                        continue
                    dir_sort_func(vec)
                    if p.axis_count == 3:
                        if p._local_config == 1:
                            ext_lay = self.import_layers[vec.lay_index]
                            gen.down_step = ext_lay.params[2]
                            gen.z_level = -gen.down_step
                            if (ext_lay.width_comp == 1) and (ext_lay.diameter != ext_lay.path_width):
                                generate_line_func = self.__generate_line_multipass3d
                                generate_arc_func = self.__generate_arc_multipass3d
                                gen.dir = ext_lay.dir
                                gen.diameter = ext_lay.diameter
                                gen.path_width = ext_lay.path_width
                            else:
                                generate_line_func = self.__generate_line3d
                                generate_corr_line_func = self.__generate_corr_line3d
                                generate_arc_func = self.__generate_arc3d

                    #local_change_config_func(vec.config_index)
                    #local_change_tool_func(int(self.machine.curr_material.configs[vec.config_index].tool))
                    if vec.type == VectorType.LINE:
                        self.__fix_inverted_vectors(n, lay, vec)
                        if (abs(gen.cx - vec.begin.x) >= p.precision) or (abs(gen.cy - vec.begin.y) >= p.precision):
 #                           if (self.params._fix_inverted_vectors == 1) and \
 #                              (distance2d(gen.cx, gen.cy, vec.end.x, vec.end.y) <
 #                              distance2d(gen.cx, gen.cy, vec.begin.x, vec.begin.y)):
 #                               vec.end.x, vec.begin.x = vec.begin.x, vec.end.x
 #                               vec.end.y, vec.begin.y = vec.begin.y, vec.end.y
 #                               vec.dir = -vec.dir
                            spindle_disable_func()
                            override_disable_func()
                            move_up_func(vec, coords)
                            generate_rapid_func(vec, coords)
                            rapid_delay_func()
                            move_zero_func(vec, coords)
                        override_enable_func()
                        for self.gen.local_curr_pass in range(local_pass_count):
                            move_down_func(vec, coords)
                            spindle_enable_func()
                            generate_line_func(vec, coords)
                            local_next_pass_func()
                        local_restore_pass_func()
                        update_curr_pos_func(vec)
                    elif vec.type == VectorType.ARC:
                        self.__fix_inverted_vectors(n, lay, vec)
                        dbxcx = abs(gen.cx - vec.begin.x)
                        dbycy = abs(gen.cy - vec.begin.y)
                        if (dbxcx >= p.precision) or (dbycy >= p.precision):
#                            if (self.params._fix_inverted_vectors == 1) and \
#                               (distance2d(gen.cx, gen.cy, vec.end.x, vec.end.y) <
#                               distance2d(gen.cx, gen.cy, vec.begin.x, vec.begin.y)):
#                                vec.end.x, vec.begin.x = vec.begin.x, vec.end.x
#                                vec.end.y, vec.begin.y = vec.begin.y, vec.end.y
#                                vec.dir = -vec.dir
                            spindle_disable_func()
                            override_disable_func()
                            move_up_func(vec, coords)
                            generate_rapid_func(vec, coords)
                            rapid_delay_func()
                            move_zero_func(vec, coords)
#                        elif (dbxcx >= EMC_PRECISION) or (dbycy >= EMC_PRECISION):
#                            generate_corr_line_func(vec, coords)
#                            move_zero_func(vec, coords)
                        override_enable_func()
                        for self.gen.local_curr_pass in range(local_pass_count):
                            move_down_func(vec, coords)
                            spindle_enable_func()
                            generate_arc_func(vec, coords)
                            local_next_pass_func()
                        local_restore_pass_func()
                        update_curr_pos_func(vec)
                global_next_pass_func()
                spindle_disable_func()
                override_disable_func()
                move_up_func(vec, coords)
        if p.spindle.mill_mode == 0:
            spindle_disable_func()
        override_disable_func()
        move_up_func(vec, coords)
        if p.axis_count == 2:
            gen.out.append(gen.g0_format % (0, 0))
            coords[len(gen.out) - 1] = [0, gen.config_index, (gen.cx, gen.cy), (0, 0)]
        elif p.axis_count == 3:
            gen.out.append(gen.g0_format % (0, 0, gen.up))
            coords[len(gen.out) - 1] = [0, gen.config_index, (gen.cx, gen.cy, gen.up), (0, 0, gen.up)]
            gen.out.append("G53 G0 Z%.4f" % gen.up)
        gen.out.append("M2")

        gen.out = filter(lambda line: line != None, gen.out)
        self.gcode = gen.out
#        self.coords = coords
        print "INFO: GENERATOR: Generating time: %d ms" % ((time.time() - t)*1000.0)
