#!/usr/bin/python2.4
# -*- coding: utf-8 -*-

#    Michal Geszkiewicz <micges@wp.pl>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from Tkinter import *
import sys, os
import string
import time
from dxf import DXF
from generator import Gen
from vector import *

class Widgets:
    def __init__(self):
        self.canvas = None
        self.left_f = None
        self.layers = None
        self.sb = None
        self.feed_lbl = None
        self.feed_entry = None
        self.feed_var = StringVar()
        self.spindle_lbl = None
        self.spindle_speed = None
        self.spindle_var = StringVar()
        
        self.move_btn = None
        self.rotate_btn = None
        self.open_btn = None
        self.label1 = None
        self.fname_entry = None
        self.fname_var = StringVar()
        self.label2 = None
        self.outname_entry = None
        self.outname_var = StringVar()
        self.gen_btn = None
        self.exit_btn = None

root = Tk()

PREVIEW_FRAME = 10

class Main(DXF, Gen):
    def __init__(self):
        DXF.__init__(self)
        Gen.__init__(self)
        root.title("vec2ngc v0.1.0 beta")
        font = "Helvetica 12"
        self.canvas_items = []
        self.canvas_cross = []
        self.curr_index = -1
        self.min = Point2d(100000.0, 100000.0)
        self.max = Point2d(-100000.0, -100000.0)
        self.layers = []

        self.w = w = Widgets()
        f = w.left_f = Frame(root)
        f.grid(column=0, row=0, sticky="nwe")
        w.layers = self.Listbox(f, 25, 8)
        w.layers.config(font=font, selectmode=SINGLE)
        w.layers.bind("<Button-1>", lambda event: self.__LayerSelectCB(event))
        w.layers.grid(column=0, row=0, columnspan=2, sticky="nwe")
        w.sb = self.Scrollbar(f)
        w.sb.grid(column=1, row=0, sticky="nse")
        w.sb.config(command=w.layers.yview)
        
        feed_lbl = self.Label(f, text="Feedrate:", font=font)
        feed_lbl.grid(column=0, row=1, columnspan=1, sticky="nse")
        feed_entry = self.Entry(f)
        feed_entry.config(font="Helvetica 14", textvariable=w.feed_var, width=12)
        feed_entry.grid(column=1, row=1, columnspan=1, sticky="nswe", pady=1)
        
        spindle_lbl = self.Label(f, text="Spindle speed:", font=font)
        spindle_lbl.grid(column=0, row=2, columnspan=1, sticky="nse")
        spindle_entry = self.Entry(f)
        spindle_entry.config(font="Helvetica 14", textvariable=w.spindle_var, width=12)
        spindle_entry.grid(column=1, row=2, columnspan=1, sticky="nswe", pady=1)
        
        w.feed_var.set(1000)
        w.spindle_var.set(1000)
        
        self.canvas = w.canvas = Canvas(root, bg="white", width=800, height=600)
        w.canvas.grid(column=1, row=0, columnspan=2)

        w.open_btn = self.Button(root, font, "Open...")
        w.open_btn.config(command=self.open_file)
        w.open_btn.grid(column=0, row=1, sticky="nwe")
        w.move_btn = self.Button(root, font, "Move to 0,0")
        w.move_btn.config(command=self.MoveTo0)
        w.move_btn.grid(column=1, row=1, sticky="nwe")
        w.rotate_btn = self.Button(root, font, "Rotate")
        w.rotate_btn.config(command=self.Rotate)
        w.rotate_btn.grid(column=2, row=1, sticky="nwe")

        w.label1 = self.Label(root, text="Input file:", font=font)
        w.label1.grid(column=1, row=2, columnspan=1, sticky="nse")
        w.fname_entry = self.Entry(root)
        w.fname_entry.config(font="Helvetica 14", textvariable=w.fname_var, justify="left")
        w.fname_entry.grid(column=2, row=2, columnspan=1, sticky="nswe", pady=2)
        
        w.label2 = self.Label(root, text="Output file:", font=font)
        w.label2.grid(column=1, row=3, columnspan=1, sticky="nse")
        w.outname_entry = self.Entry(root)
        w.outname_entry.config(font="Helvetica 14", textvariable=w.outname_var, justify="left")
        w.outname_entry.grid(column=2, row=3, columnspan=1, sticky="nswe", pady=2)
        
        w.gen_btn = self.Button(root, font, "Generate")
        w.gen_btn.config(command=self.generate)
        w.gen_btn.grid(column=1, row=4, sticky="nwe")
        w.exit_btn = self.Button(root, font, "Exit")
        w.exit_btn.config(command=self.exit)
        w.exit_btn.grid(column=2, row=4, sticky="nwe")

        w.outname_var.set("gcode.ngc")

        p = self.params
        p.axis_count = 3
        p.axis_names = "XYZ"
        p.out_precision_fields = 4
        p.precision = 0.01

        p.configs.change_gcode = ""

        p.rapid_delay.control = 1
        p.rapid_delay.gcode = "G4 P0"

        p.override.control = 1
        p.override.enable_gcode = "M50 P1"
        p.override.disable_gcode = "M50 P0"

        p.spindle.mill_mode = 1
        p.spindle.enable_gcode = "M4"
        p.spindle.enable_delay_gcode = "G4 P3"
        p.spindle.disable_gcode = "M5"
        p.spindle.disable_delay_gcode = "G4 P3"

        p._tool.control = 0
        p._tool.changer = 0

    def open_file(self):
        all_extensions = tuple([".dxf"])
        types = (("DXF files", "*.dxf"),)
        f = root.tk.call("tk_getOpenFile", "-initialdir", "./",
            "-filetypes", types)
        if not f: 
            return
        self.file_name = str(f)
        self.w.fname_var.set(self.file_name)

        f = open(self.file_name)
        self.source = []
        self.source = f.read()
        self.source = self.source.split("\n")
        f.close()

        self.dxf_load_file()
        
        self.w.layers.delete(0, END)
        for lay in self.layers:
            self.w.layers.insert(END, lay.name)
            lay.params.append(5)
            lay.params.append(1)
            lay.params.append(0)
            lay.params.append(1)
            
        self.Draw(-1)

    def main(self):
        root.mainloop()
        
    def generate(self):
        p = self.params
        p._base_velocity = int(self.w.feed_var.get())
        p._spindle_velocity = int(self.w.spindle_var.get())

        self.generate_gcode(self.curr_index)
        path = "./" + self.w.outname_var.get()
        f = open(path, "w+")
        for i in self.gcode:
            f.write("%s\n" % i)
        f.close()
        
    def exit(self):
        root.destroy()

    def Label(self, master, font, text):
        return Label(master, font=font, text=text)

    def Scrollbar(self, master):
        return Scrollbar(master, takefocus=0, bd=1, elementborderwidth=1)

    def Button(self, master, font, text):
        return Button(master, takefocus=0, bd=1, text=text, font=font,
          disabledforeground="darkgrey")

    def Listbox(self, master, width, height):
        return Listbox(master, takefocus=0, bd=0,
          width=width, height=height, highlightbackground="#888888", exportselection=0,
          selectborderwidth=0, selectbackground="black", selectforeground="white")

    def Entry(self, master):
        return Entry(master, takefocus=0, bd=0, justify="right",
          highlightbackground="#888888", disabledforeground="black", exportselection=0)

    def __LayerSelectCB(self, event):
        self.curr_index = int(self.w.layers.index("@%d,%d" % (event.x, event.y)))
        self.Draw(self.curr_index)

    def Draw(self, selected):
        canvas = self.canvas

        canvas.delete(*self.canvas_items)
        self.canvas_items = []
        canvas.delete(*self.canvas_cross)
        self.canvas_cross = []

        t = time.time()
        minx, miny = 0.0, 0.0
        if (self.min.x < 0):
            minx = self.min.x
        if (self.min.y < 0):
            miny = self.min.y
        maxx, maxy = self.max.x, self.max.y
        if (self.max.x == 0):
            maxx = 1
        if (self.max.y == 0):
            maxy = 1

        canvasw = float(canvas.cget("width"))
        canvash = float(canvas.cget("height"))
        sx = (canvasw - PREVIEW_FRAME*2)/abs(maxx - minx)
        sy = (canvash - PREVIEW_FRAME*2)/abs(maxy - miny)
        if sx < sy:
            scale = sx
        else:
            scale = sy

        scminx = minx*scale
        scminy = miny*scale
        selected_layer = None

        for i, lay in enumerate(self.layers):
            if i == selected:
                selected_layer = lay
                continue
            for vec in lay.vectors:
                if vec.type == VectorType.LINE:
                    self.canvas_items.append(canvas.create_line(
                      vec.begin.x, vec.begin.y, vec.end.x, vec.end.y, fill="black"))
                    canvas.scale(self.canvas_items[-1], 0, 0, scale, -scale)
                    canvas.move(self.canvas_items[-1], PREVIEW_FRAME - scminx,
                      canvash + scminy)
                elif vec.type == VectorType.ARC:
                    x0 = vec.center.x - vec.radius
                    x1 = vec.center.x + vec.radius
                    y0 = vec.center.y - vec.radius
                    y1 = vec.center.y + vec.radius
                    self.canvas_items.append(canvas.create_arc(x0, y0, x1, y1,
                      start=vec.view.start_angle, extent=vec.view.angle, style=ARC, outline="black"))
                    canvas.scale(self.canvas_items[-1], 0, 0, scale, -scale)
                    canvas.move(self.canvas_items[-1], PREVIEW_FRAME - scminx,
                      canvash + scminy)
                elif vec.type == VectorType.TEXT:
                    self.canvas_items.append(canvas.create_text(vec.begin.x, vec.begin.y,
                      text=vec.text))
                    canvas.scale(self.canvas_items[-1], 0, 0, scale, -scale)
                    canvas.move(self.canvas_items[-1], PREVIEW_FRAME - scminx,
                      canvash + scminy)
        if selected_layer != None:
            for vec in selected_layer.vectors:
                if vec.type == VectorType.LINE:
                    self.canvas_items.append(canvas.create_line(
                      vec.begin.x, vec.begin.y, vec.end.x, vec.end.y, fill="blue"))
                    canvas.scale(self.canvas_items[-1], 0, 0, scale, -scale)
                    canvas.move(self.canvas_items[-1], PREVIEW_FRAME - scminx,
                      canvash + scminy)
                elif vec.type == VectorType.ARC:
                    x0 = vec.center.x - vec.radius
                    x1 = vec.center.x + vec.radius
                    y0 = vec.center.y - vec.radius
                    y1 = vec.center.y + vec.radius
                    self.canvas_items.append(canvas.create_arc(x0, y0, x1, y1,
                      start=vec.view.start_angle, extent=vec.view.angle, style=ARC, outline="blue"))
                    canvas.scale(self.canvas_items[-1], 0, 0, scale, -scale)
                    canvas.move(self.canvas_items[-1], PREVIEW_FRAME - scminx,
                      canvash + scminy)
                elif vec.type == VectorType.TEXT:
                    self.canvas_items.append(canvas.create_text(vec.begin.x, vec.begin.y,
                      text=vec.text, fill="blue"))
                    canvas.scale(self.canvas_items[-1], 0, 0, scale, -scale)
                    canvas.move(self.canvas_items[-1], PREVIEW_FRAME - scminx,
                      canvash + scminy)

        self.canvas_cross.append(
          canvas.create_line(-PREVIEW_FRAME, 0, PREVIEW_FRAME, 0, fill="red"))
        canvas.scale(self.canvas_cross[-1], 0, 0, scale, -scale)
        canvas.move(self.canvas_cross[-1], PREVIEW_FRAME  - scminx, canvash + scminy)
        self.canvas_cross.append(
          canvas.create_line(0, -PREVIEW_FRAME, 0, PREVIEW_FRAME, fill="red"))
        canvas.scale(self.canvas_cross[-1], 0, 0, scale, -scale)
        canvas.move(self.canvas_cross[-1], PREVIEW_FRAME  - scminx, canvash + scminy)

        print "INFO: Czas generowania podglądu: %d ms" % ((time.time() - t)*1000.0)

    def MoveTo0(self):
        if self.min.x < self.max.x:
            dx = -self.min.x
        else:
            dx = -self.max.x
        if self.min.y < self.max.y:
            dy = -self.min.y
        else:
            dy = -self.max.y
        self.min.x += dx
        self.min.y += dy
        self.max.x += dx
        self.max.y += dy
        for i in self.layers:
            for j in i.vectors:
                j.begin.x += dx
                j.begin.y += dy
                j.end.x += dx
                j.end.y += dy
                j.center.x += dx
                j.center.y += dy
        self.Draw(self.curr_index)

    def Rotate(self):
        self.min.x, self.min.y = self.min.y, -self.min.x
        self.max.x, self.max.y = self.max.y, -self.max.x
        if self.min.x > self.max.x:
            self.min.x, self.max.x = self.max.x, self.min.x
        if self.min.y > self.max.y:
            self.min.y, self.max.y = self.max.y, self.min.y
        for i in self.layers:
            for j in i.vectors:
                j.begin.x, j.begin.y = j.begin.y, -j.begin.x
                j.end.x, j.end.y = j.end.y, -j.end.x
                j.center.x, j.center.y = j.center.y, -j.center.x
                j.view.start_angle -= 90
        self.Draw(self.curr_index)

main = Main()
main.main()

