#!/usr/bin/python2.4
# -*- coding: utf-8 -*-

from math import *
from base_import import *

PI_2 = pi/2.0
EMC_PRECISION = 0.0001

class InputFormat:
    NONE = 0
    DXF = 1
    CF2 = 2
    PLT = 3
    ISO = 4

class ViewInfo:
    def __init__(self):
        self.start_angle = 0.0
        self.angle = 0.0

class Point2d:
    def __init__(self, x=0.0, y=0.0):
        self.x = x
        self.y = y

class Point3d:
    def __init__(self, x=0.0, y=0.0, z=0.0):
        self.x = x
        self.y = y
        self.z = z

class Vertex(Point3d):
    def __init__(self, x=0.0, y=0.0, b=0.0):
        Point3d.__init__(self, x, y)
        self.bulge = b

class Vector:
    def __init__(self, bx=0.0, by=0.0, ex=0.0, ey=0.0, scale=1.0):
        self.type = VectorType.NONE
        self.begin = Point3d(bx*scale, by*scale)
        self.end = Point3d(ex*scale, ey*scale)
        self.center = Point3d()
        self.start_angle = 0.0
        self.end_angle = 0.0
        self.angle = 0.0
        self.radius = 0.0
        self.dir = 0
        self.width = -1.0
        self.bridges_count = 0
        self.bridges_width = 0.0
        self.index = 0
        self.config_index = None
        self.lay_index = None
        self.text_height = 0.0
        self.text_rotate = 0.0
        self.text = ""
        self.view = ViewInfo()

class SourceLayer:
    def __init__(self):
        self.name = ""
        self.vectors = []
        self.params = []
        self.gcode = []
        self.config_index = -1
        self.gcode_len = -1
        self.coords = None
        self.config_index = -1
        self.tool = -1
        self.multipass_type = MultipassTypeEnum.GLOBAL
        self.sort = 0
        self.width_comp = 0
        self.dir = 0
        self.diameter = 0
        self.path_width = 0


def distance2d(x1, y1, x2, y2):
    return (((x2 - x1)**2 + (y2 - y1)**2)**0.5)

def find_angle(x1, y1, x2, y2, x3, y3):
    Angle = 0.0
    dx = 0.0
    dr = 0.0

    if ((abs(x2 - x3) < 0.01) and (y2 < y3)): # x0, y0 -> y+
        Angle = 90.0
    elif ((x2 > x3) and (abs(y2 - y3) < 0.01)): # x0 -> x-, y0
        Angle = 180.0
    elif ((abs(x2 - x3) < 0.01) and (y2 > y3)): # x0, y0 -> y-
        Angle = 270.0
    elif ((x2 < x3) and (abs(y2 - y3) < 0.01)): # x0 -> x+, y0
        Angle = 360.0
    elif ((x2 < x3) and (y2 < y3)): # 1 cwiartka
        dx = x3 - x2
        dr = distance2d(x2, y2, x3, y3)
        Angle = degrees(acos(dx/dr))
    elif ((x2 > x3) and (y2 < y3)): # 2 cwiartka
        dx = abs(x3 - x2)
        dr = distance2d(x2, y2, x3, y3)
        Angle = 180.0 - degrees(acos(dx/dr))
    elif ((x2 > x3) and (y2 > y3)): # 3 cwiartka
        dx = abs(x3 - x2)
        dr = distance2d(x2, y2, x3, y3)
        Angle = degrees(acos(dx/dr)) + 180.0
    elif ((x2 < x3) and (y2 > y3)): # 4 cwiartka
        dx = abs(x3 - x2)
        dr = distance2d(x2, y2, x3, y3)
        Angle = 360.0 - degrees(acos(dx/dr))
    return Angle

def create_arc_view_data(v):
    bangle = find_angle(v.center.x + 1, v.center.y, v.center.x, v.center.y, v.begin.x, v.begin.y)
    eangle = find_angle(v.center.x + 1, v.center.y, v.center.x, v.center.y, v.end.x, v.end.y)
    if (abs(bangle - eangle) < 0.01):
        v.view.angle = 359.9
        v.view.start_angle = 0
    elif (bangle < eangle) and v.dir > 0:
        v.view.angle = (eangle - bangle) - 360.0
        v.view.start_angle = bangle
    elif (bangle < eangle) and v.dir < 0:
        v.view.angle = (eangle - bangle)*v.dir
        v.view.start_angle = eangle
    elif (bangle > eangle) and v.dir > 0:
        v.view.angle = (bangle - eangle)*v.dir
        v.view.start_angle = eangle
    elif (bangle > eangle) and v.dir < 0:
        v.view.angle = (bangle - eangle) - 360.0
        v.view.start_angle = eangle
